import { useRef } from 'react';
// @ts-ignore

export default function useFileUpload(_ref) {
  let {
    url,
    field,
    method = 'POST',
    headers,
    timeout,
    onProgress,
    onDone,
    onError,
    onTimeout
  } = _ref;
  const requests = useRef({});
  const startUpload = item => {
    return new Promise((resolve, reject) => {
      const formData = new FormData();
      formData.append(field, item);
      const xhr = new XMLHttpRequest();
      xhr.open(method, url);
      requests.current[item.uri] = xhr;
      if (xhr.upload) {
        xhr.upload.onprogress = event => {
          onProgress === null || onProgress === void 0 ? void 0 : onProgress({
            item,
            event
          });
        };
      }
      if (timeout) {
        xhr.timeout = timeout;
        xhr.ontimeout = () => {
          const result = {
            item,
            error: xhr.responseText,
            timeout: true
          };
          onTimeout === null || onTimeout === void 0 ? void 0 : onTimeout(result);
          reject(result);
        };
      }
      xhr.onload = () => {
        const result = {
          item,
          responseBody: xhr.response || xhr.responseText,
          responseHeaders: xhr.getAllResponseHeaders()
        };
        onDone === null || onDone === void 0 ? void 0 : onDone(result);
        resolve(result);
      };
      xhr.onerror = () => {
        const result = {
          item,
          error: xhr.responseText
        };
        onError === null || onError === void 0 ? void 0 : onError(result);
        reject(result);
      };
      headers === null || headers === void 0 ? void 0 : headers.forEach((value, key) => {
        xhr.setRequestHeader(key, value);
      });
      xhr.send(formData);
    });
  };
  const abortUpload = uri => {
    var _requests$current$uri;
    (_requests$current$uri = requests.current[uri]) === null || _requests$current$uri === void 0 ? void 0 : _requests$current$uri.abort();
  };
  return {
    startUpload,
    abortUpload
  };
}
//# sourceMappingURL=useFileUpload.js.map